/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2008, 2011 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
 
/**
 * ContextMenu object.  Provides the basic communication to send request to a servlet and handle the response.
 */
function USMiContextMenu(contextPath, contextMenuId, dualListId, nameSpacedReceiveMenu) { 
  this.contextPath = contextPath;
  this.contextMenuId = contextMenuId;
  this.nameSpacedReceiveMenu = nameSpacedReceiveMenu;

  this.dualListParam = '';  
  if (null != dualListId && dualListId.length > 0) {
    this.dualListParam = '&dualListChooserInputField='+dualListId;
  }

  this.menuRequest = null;

  // Generate a reusable behavior that we can configure for each menu item that is selected
  this.reusableMenuBehavior = new hX_6.JSFBehaviorGeneric();
  hX_6.addBehavior(contextMenuId, 'onclick', this.reusableMenuBehavior);

  // hard coded dimensions - since this isn't a customizable value doesn't see a need to keep it on the renderer
  contextMenuSetArrowIconDimensions('12', '12');
}

/**
 * Requests the menu to the Servlet. 
 */
USMiContextMenu.prototype.requestMenu = function (event, menuItem, menuId, otherParams) {

  waitState(true);
  
  var eventTarget = null;
  if (null != event) {
    eventTarget = getEventTarget(event);
    if (null != eventTarget && eventTarget.id == "" && eventTarget.className == "searchword") {
      eventTarget = eventTarget.parentNode;
    }
  }
  
  var menuSourceId = (menuId ? menuId : eventTarget.id);

  var sourceClientId = menuSourceId + (menuItem?('&menuItem='+menuItem) : '') + '&contextMenuId='+this.contextMenuId + this.dualListParam +(otherParams?('&'+ otherParams) : '');

  this.sendRequest(sourceClientId);

  /** 
   * NOTE: In regard to event.cancelBubble=true...Returning false cancels the event, which is supposed 
   * to stop it from "bubbling", so nothing else happens.  However, according to 
   * http://www.webreference.com/js/column10/cancel.html, there are some slightly 
   * different implications in IE of returning false and cancelBubble.  Frankly, I don't really 
   * understand how ILOG could be getting the event after we return false, but cancelBubble stops 
   * it from happening.  I think it may be a bug in IE.
   */
  if (event) {
    event.cancelBubble=true;
  }
  return false;
}

/**
 * Creates a request object.
 */
USMiContextMenu.prototype.createRequestObject = function() {

  if (hX_6.brw.isIE()) {
    this.menuRequest = new ActiveXObject('Microsoft.XMLHTTP');
  } else {
    this.menuRequest = new XMLHttpRequest();
  }

  //NOTE: on both IE and FF the onreadystatechange needs to be attach for every request
  this.menuRequest.onreadystatechange = this.nameSpacedReceiveMenu;
}

/**
 * Sends an ajax http request to the ContextMenuServlet.
 */
USMiContextMenu.prototype.sendRequest = function (sourceClientId) {

  //on IE the requestObject needs to be created for each request, doing for both IE and FF
  this.createRequestObject();
  this.menuRequest.open('GET', this.contextPath + '/ContextMenuServlet?sourceClientId=' + sourceClientId, true);
  this.menuRequest.send(null);

}

/**
 * Handles the response from the requests.
 * Response should be formatted javascript that can be passed to the eval method.
 */
USMiContextMenu.prototype.receiveMenu = function () {

  if(this.menuRequest.readyState != 4) {
    return;
  }

  if(this.menuRequest.status != 200) {
    alert("There was a problem handling the request. See the logs for details.");
    return;
  }

  /**
   * the responseText is a bunch of javascript that is ready to execute; it will create & show the menu
   * see the servlet code for details  
   */
  eval(this.menuRequest.responseText);
  waitState(false);
}

