/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2008, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/**
*	Extending HxG_6
*/

HxG_6.prototype.registerEdt = function(edtInstance)
{
	if(this.registeredEdts == null)
	{
		this.registeredEdts = new Object();
	}
	this.registeredEdts[edtInstance.tableId]=edtInstance;
}

/**
* We could have used the JWL event mecanism to do this 
* but broadcasting events using JWL is too slow in IE.
*/
HxG_6.prototype.broadcastEdtInit = function(waitPeriod)
{
	if(waitPeriod)
	{
		//if there is wait period call the method after the period has
		//expired.
		setTimeout("hX_6.broadcastEdtInit();", waitPeriod * 1000);
	}
	else
	{
		for(var edtId in this.registeredEdts)
		{
			//check if the edt is still in the DOM
			if(document.getElementById(edtId) != null)
			{
				var edtInstance = this.registeredEdts[edtId];
				edtInstance.init();
			}
			else
			{
				//edt no longer in the DOM (probably removed by ajax) so lets unregister it
				this.registeredEdts[edtId] = null;
			}
		}
	}
}

/**
* In portal sometimes the HxG_6 library is added more than once therefor
* we need to make sure that the methods that we added were not overwritten
* when a new definition of the class HxG_6 is declared.
* If it is then then we need to add them to the global instance.
*/
if (hX_6 && hX_6.bodyLoadRewritten) 
{
 	if (HxG_6.prototype.registerEdt && !hX_6.registerEdt) 
 	{
 		hX_6.registerEdt = HxG_6.prototype.registerEdt;
 	 	hX_6.broadcastEdtInit = HxG_6.prototype.broadcastEdtInit;
 	}
}

/*
 * Defining table JS object & related methods
 */
function HxDataTableEx(elementId, exploiterInit, exploiterOnAjaxComplete, hideScrollbar, useEllipses, gotoId, toolBarId, isAccessible, emptyTableMessage, isRTL, rowCount, selectedRowStyle) {

	this.tableId = elementId;
	this.exploiterInit = exploiterInit;
	this.exploiterOnAjaxComplete = exploiterOnAjaxComplete;
	this.hideScrollbar = hideScrollbar;
	this.useEllipses = useEllipses;
	this.gotoId = gotoId;
	this.isAccessible = isAccessible;
	this.emptyTableMessage = emptyTableMessage;
	this.isRTL = isRTL;
	this.toolBarId = toolBarId;
	this.previousScrollSize = 0;
	this.rowCount = rowCount;
	//Set table title if it is empty table.
	 if(this.rowCount==0){
	    this.turnOnEmptyTableMessage();
  	 }else{
	 	this.turnOffEmptyTableMessage();
	 }

	this.selectedRowStyle = selectedRowStyle;
	
	//flag that determines which algorithm to use on IE to calcualte ellipses
	this.useEllipsesAprox = false;

	this.variableName = "hx_" + this.tableId.replace(/[^a-zA-Z0-9]/g,'_') + "__jsDataTable";	

	//register to receive init events.
	hX_6.registerEdt(this);
}

HxDataTableEx.prototype.onPageLoad = function() {
  this.entering('HxDataTableEx.prototype.onPageLoad');
  
  this.adjustBothScrolls(true);	

  if (hX_6.brw.isIE()) {

    //problem with the pageControls images beings mis aligned on IE
    //printlnTable("\n redrawing on IE");
    this.refreshPagerGoto();
    //the following line is causing defect 91920. I inspected the table
    //visually and this call seems not to be making any difference
    // hX_6.redraw();		
  }
  this.initMouseWheel();	
  this.highlightSelectedRows();
  
    //call exploiter init method
  if(this.exploiterInit != null)
  {
 	 try
  	{
  		this.exploiterInit(this);
 	}
  	catch(e)
  	{
  		alert("ERROR calling custom initialization script.");
  	}
  }
  this.exiting('HxDataTableEx.prototype.onPageLoad');
  return true;
}

HxDataTableEx.prototype.onResize = function() {
	//printlnTable("\nHxDataTableEx.prototype.onResize");
	this.adjustBothScrolls(false);
}

HxDataTableEx.prototype.onAjaxComplete = function() {
	//printlnTable("\nHxDataTableEx.prototype.onAjaxComplete");
	this.onDrawChangeComplete();
	this.clearHiddenIds();
	
	 //call exploiter method
  	if(this.exploiterOnAjaxComplete != null)
  	{
 	 	try
  		{
  			this.exploiterOnAjaxComplete(this);
 		}
  		catch(e)
  		{
  			alert("ERROR calling custom on ajax complete script.");
  		}
 	 }
}

//call after doing DOM manipulation in the rows of the table
HxDataTableEx.prototype.onDrawChangeComplete = function() {
	//printlnTable("\nHxDataTableEx.prototype.onDrawChangeComplete");
	this.onResize();
	this.highlightSelectedRows();
}

HxDataTableEx.prototype.init = function() {
	this.adjustBothScrolls(false);	
	//clear aprox vars
  if (this.useEllipsesAprox && hX_6.brw.isIE()) {
    this.clearAproxVars();
  }	 
	return;	
}


HxDataTableEx.prototype.clearHiddenIds = function() {
  if (typeof hx_jsHidden != "undefined") {
    hx_jsHidden.clearHiddenIds();
  }
}

HxDataTableEx.prototype.getScrollHeight = function(divId, scrollSize) {
	var div = document.getElementById(divId);
	if (div.scrollHeight > scrollSize) {
		return scrollSize;
	} else {
		return div.scrollHeight;
	}
}

HxDataTableEx.prototype.turnOffEmptyTableMessage = function() {
	var mainDiv = document.getElementById(this.tableId);
	mainDiv.title = "";
	
	if(this.isAccessible)
	{
		var rightTable = document.getElementById(this.tableId + "Right");
		var rightHeaderTable = document.getElementById(this.tableId + "RightColumnheader");
		
		if(rightHeaderTable != null)
		{
			rightTable.summary  = rightHeaderTable.summary;
		}
		else
		{
			rightTable.summary="";
		}
	}
	
}

HxDataTableEx.prototype.turnOnEmptyTableMessage = function() {
	var mainDiv = document.getElementById(this.tableId);
	mainDiv.title = this.emptyTableMessage;
	
	if(this.isAccessible)
	{
		var rightTable = document.getElementById(this.tableId + "Right");
		if (rightTable) {
			rightTable.summary = this.emptyTableMessage;
		}
	}
	
}

HxDataTableEx.prototype.onHorizontalScroll = function() {

	if(!this.tableId) {
	  return;
	}

	var objDiv = hX_6.getElementById(this.tableId+"hScrollOuter");	
	if (!objDiv) {
	  return;
	}

	var leftValue = objDiv.scrollLeft;

	var rightH = hX_6.getElementById(this.tableId+"columnheaderRightCellsDiv");
	var rightC = hX_6.getElementById(this.tableId+'RightBodyDiv');
	var rightF = hX_6.getElementById(this.tableId+"columnfooterRightCellsDiv");	

	if (rightH) rightH.scrollLeft = leftValue;
	if (rightC) rightC.scrollLeft = leftValue;
	if (rightF) rightF.scrollLeft = leftValue;	

	return;
}

HxDataTableEx.prototype.onHeaderHorizontalScroll = function() {

	if(!this.tableId) {
	  return;
	}

	var objDiv = hX_6.getElementById(this.tableId+"columnheaderRightCellsDiv");	
	if (!objDiv) {
	  return;
	}

	var leftValue = objDiv.scrollLeft;

	var rightC = hX_6.getElementById(this.tableId+'RightBodyDiv');
	var rightF = hX_6.getElementById(this.tableId+"columnfooterRightCellsDiv");	
	var rightS = hX_6.getElementById(this.tableId+"hScrollOuter");

	if (rightC) rightC.scrollLeft = leftValue;
	if (rightF) rightF.scrollLeft = leftValue;
	if (rightS) rightS.scrollLeft = leftValue;

	return;
}

HxDataTableEx.prototype.onVerticalScroll = function() {

	if(!this.tableId) {
	  return;
	}

	var objDiv = hX_6.getElementById(this.tableId+"vScrollOuter");
	if(!objDiv) {
	  return;	
	}

	var topValue = objDiv.scrollTop;

	var leftC = hX_6.getElementById(this.tableId+"LeftBodyDiv");
	var rightC = hX_6.getElementById(this.tableId+"RightBodyDiv");
	if (leftC) leftC.scrollTop = topValue;
	if (rightC) rightC.scrollTop = topValue;
	
	return;
}			

HxDataTableEx.prototype.adjustVerticalScroll = function() {
  this.entering('HxDataTableEx.prototype.adjustVerticalScroll');

	if(!this.tableId) {
		return;
	}
	
	/*********** adjusting scroll sizes ************/	

	/********** vertical scroll *******************/
	var outerScroll = hX_6.getElementById(this.tableId+'vScrollOuter');
	var innerScroll = hX_6.getElementById(this.tableId+'vScrollInner');
	
	if(!outerScroll || !innerScroll) {
		return;
	}

	var tableDivLeft = hX_6.getElementById(this.tableId+'LeftBodyDiv');	
	var tableDivRight = hX_6.getElementById(this.tableId+'RightBodyDiv');	
	
	if(!tableDivLeft && !tableDivRight) {
		return;
	}

	var diff = 0;
	if (tableDivRight) {
		diff = tableDivRight.scrollHeight - tableDivRight.clientHeight;
	} else if (tableDivLeft) {
		diff = tableDivLeft.scrollHeight - tableDivLeft.clientHeight;			
	}
	
	//printlnTable(tableDivRight.scrollHeight+' - '+tableDivRight.clientHeight+' & '+tableDivRight.offsetHeight);
	//printlnTable(tableDivLeft.scrollHeight+' - '+tableDivLeft.clientHeight+' & '+tableDivLeft.offsetHeight);	
	
	var mtable = hX_6.getElementById(this.tableId+'MainBodyTable');
	var mcols = mtable.getElementsByTagName('colgroup')[0].getElementsByTagName('col');
	
	if (!mtable || !mcols) {
		return;
	}

	if (diff > 0 || !this.hideScrollbar) {

		//alert('rows: '+mtable.rows.length);
		var hi = 0;
		//alert('number of rows: '+mtable.rows.length);
		for(var j=0; j < mtable.rows.length; j++) {
			//alert('cells: '+mtable.rows[j].cells.length);
			//if (j == 0) continue;
			if (j == (mtable.rows.length -1) && hX_6.getElementById(this.tableId+'hScrollOuter')) continue;
			hi += mtable.rows[j].offsetHeight;
			
		}
		outerScroll.style.height = hi + 'px';
		//alert(diff+" "+outerScroll.clientHeight);
		innerScroll.style.height = hi + diff + 'px';
		var scrollbarWidth = 0;
		scrollbarWidth = this.getScrollbarWidth(mtable);
		//In IE vertical scrollbar is not working since width is less than
		// desktop scroller width
		if(hX_6.brw.isIE()){
			scrollbarWidth = scrollbarWidth + 2;
		}
		
		mcols[mcols.length-1].style.width = scrollbarWidth + 'px';
		innerScroll.style.width = scrollbarWidth + 'px';
				
		if (!hX_6.brw.isIE() || diff == 0) {
			//FF doesn't like auto and IE doesn't like scroll ;-)
			outerScroll.style.overflowY = "scroll";
			outerScroll.style.overflowX = "hidden";
			if (!this.hideScrollbar && diff == 0) {
				innerScroll.style.height = 0 + 'px';
			}
		}
		
		/* awa tree_table patch for IE and tree tables  begin */
		if (hX_6.brw.isIE() && diff > 0)
		{
			outerScroll.style.overflowY = "auto";
		}
		/* awa tree_table patch for IE and tree tables  begin */
		
		outerScroll.style.width =  scrollbarWidth + 'px';
		
		//outerScroll.parentNode.style.visibility="visible";
		//outerScroll.parentNode.style.display = "none";
		outerScroll.parentNode.style.display="";
		mcols[mcols.length-1].style.display = "none";
		mcols[mcols.length-1].style.display = "";	
		
	}	else {
		//outerScroll.parentNode.style.visibility="collapse";		
		mcols[mcols.length-1].style.width = '1px';
		//outerScroll.parentNode.style.display="";
		outerScroll.parentNode.style.display = "none";
		mcols[mcols.length-1].style.display = "";
		mcols[mcols.length-1].style.display = "none";
		mcols[mcols.length-1].style.display = "";		
	}
	
  this.exiting('HxDataTableEx.prototype.adjustVerticalScroll');
  return;
}

HxDataTableEx.prototype.adjustHorizontalScroll = function(isOnPageLoad) {
  
  this.entering('HxDataTableEx.prototype.adjustHorizontalScroll');

	if(!this.tableId) {
		return;
	}
	
	/********** horizontal scroll *******************/
	var outerScroll = hX_6.getElementById(this.tableId+'hScrollOuter');
	var innerScroll = hX_6.getElementById(this.tableId+'hScrollInner');

	if(!outerScroll || !innerScroll) {
		//printlnTable('\n no outerScroll or no innerScroll');	
		return;	
	}
	var tableDivLeft = hX_6.getElementById(this.tableId+'LeftBodyDiv');	
	var tableDivRight = hX_6.getElementById(this.tableId+'RightBodyDiv');	
	var diff = 0;
	
	//if(!tableDivLeft || !tableDivRight) {}
	if(!tableDivRight) {	
		//Adjust scroll on emty table
		this.adjustScrollOnEmptyData();
		return;	
	}

	


	if (tableDivRight) {
		/*
		var tableDiv = hX_6.getElementById(this.tableId);
		var chlfcd = document.getElementById('form1:jobsTablecolumnheaderLeftCellsDiv');
		var vso = document.getElementById('form1:jobsTablevScrollOuter');
		*/
		diff = tableDivRight.scrollWidth - tableDivRight.offsetWidth;//tableDivRight.clientWidth;
		//printlnTable('\ndiff: '+diff+" tdrS: "+tableDivRight.scrollWidth+" tdrC: "+tableDivRight.offsetWidth);
		
		/*
		var tableDivW = tableDiv.offsetWidth;
		var chlfcdW = chlfcd.offsetWidth;
		var vsoW = vso.offsetWidth;
		printlnTable("\nt: "+tableDivW+" h: "+chlfcdW+" vScroll: "+vsoW);
		if (vsoW == 0) {
			printlnTable("\nfirst time!");
		}
		*/
		//diff -= 17;
		if (!isOnPageLoad) {
			//var scrollbarWidth = this.getScrollbarWidth(tableDivRight);
			//diff -= scrollbarWidth;
			//printlnTable("\n isOnPageLoad: "+isOnPageLoad);
		}
	}

	if (diff > 0 || !this.hideScrollbar) {

		outerScroll.parentNode.parentNode.style.display = "";
		//outerScroll.parentNode.parentNode.style.visibility="visible";
		
		var wi = 0;
		if (tableDivLeft) {
			wi += tableDivLeft.offsetWidth;
		}

		if (tableDivRight) {
			wi += tableDivRight.offsetWidth;
		}

		outerScroll.style.width = wi + "px";
		innerScroll.style.width = ((wi + diff) + "px");
		
		var scrollbarWidth = this.getScrollbarWidth(tableDivRight);
		//printlnTable("scrollbarWidth: "+scrollbarWidth);
		outerScroll.style.height = scrollbarWidth + 'px';
		innerScroll.style.height = scrollbarWidth + 'px';
		
		if (!this.hideScrollbar && diff == 0) {
			outerScroll.style.overflowX = "scroll";
			outerScroll.style.overflowY = "hidden";
			innerScroll.style.width = 0;
		}
	
	}	else {
		outerScroll.parentNode.parentNode.style.display = "none";
		//outerScroll.parentNode.parentNode.style.visibility="collapse";		

	}
	
  this.exiting('HxDataTableEx.prototype.adjustHorizontalScroll');	
	return;
	
}

HxDataTableEx.prototype.adjustIE = function() {
	return;
	if (!hX_6.brw.isIE()) return;
	//printlnTable("HxDataTableEx.prototype.adjustIE on IE "+this.tableId);
	if (this.tableId) {
		var ieTable = document.getElementById(this.tableId);
		var ieTableWidth = ieTable.clientWidth;
		//printlnTable("ieTable: "+ieTable+" ieTableWidth: "+ieTableWidth);
		var ieLeftColumnHeader = document.getElementById(this.tableId+"columnheaderLeftCellsDiv");
		var ieLeftColumnHeaderWidth = ieLeftColumnHeader.clientWidth;
		//printlnTable("ieLeftColumnHeader: "+ieLeftColumnHeader+" ieLeftColumnHeaderWidth: "+ieLeftColumnHeaderWidth);		
		//clientId + "columnfooterLeftCellsDiv";
		//clientId + "LeftBodyDiv";
		var ieVScroll = document.getElementById(this.tableId+"vScrollOuter");
		var ieVSCrollWidth = 0;
		if (ieVScroll) ieVScrollWidth = ieVScroll.clientWidth;
		//printlnTable("ieVScroll: "+ieVScroll+" ieVScrollWidth: "+ieVScrollWidth);		
		//printlnTable("tableWidth: "+ieTableWidth+" leftHeaderWidth: "+ieLeftColumnHeaderWidth+" vScrollWidth: "+ieVScrollWidth);
		var ieRightBodyDiv = document.getElementById(this.tableId+"RightBodyDiv");
		//printlnTable("rigthBodyDiv: "+ieRightBodyDiv);
		if (hX_6.brw.isIE()) {
			if (ieRightBodyDiv) {
				//ieRightBodyDiv.style.width = ieTableWidth - ieLeftColumnHeaderWidth - ieVScrollWidth + 'px';
			}
		}
	}
}

HxDataTableEx.prototype.adjustBothScrolls = function(isOnPageLoad) {
  this.entering('HxDataTableEx.prototype.adjustBothScrolls');

	this.adjustTableRowHeights();
	this.adjustHorizontalScroll(isOnPageLoad);
	this.adjustVerticalScroll();
	this.addTitles();
	//Refresh Toolbar,toolbar was misaligned in AUIML application
	if (hX_6.brw.isIE()) {
		var toolBar = hX_6.getElementById(this.toolBarId);
    	if (toolBar)
	    	this.refreshElement(toolBar); 	
	}
	
  this.exiting('HxDataTableEx.prototype.adjustBothScrolls');	
	return;	
}

HxDataTableEx.prototype.getScrollbarWidth = function(formChild) {

	var form1=hX_6.getParentOfType(formChild, hX_6._BBODY);
	var pDiv=document.createElement("div");
	var cDiv=document.createElement("div");
	form1.appendChild(pDiv);
	pDiv.appendChild(cDiv);
		
	var pDivStyle = pDiv.style;
	pDivStyle.height="100";
	pDivStyle.width="100";
	pDivStyle.overflow="scroll";

	var cDivStyle = cDiv.style;
	cDivStyle.height="1000";
	cDivStyle.width="1000";

	var width = pDiv.offsetWidth - pDiv.clientWidth;
	form1.removeChild(pDiv);
	return width;
}

HxDataTableEx.prototype.adjustRowHeights = function(table1, table2) {  
  this.entering('HxDataTableEx.prototype.adjustRowHeights');

  var rows1 = table1.rows;
  var rows1Len = rows1.length;
  var rows2 = table2.rows;
  var i;
  var MIN_HEIGHT = 1;
	var cells1Height = null;
  var cells2Height = null;
  
	for (i = 0; i < rows1Len; i++) {
	
		if (!rows1[i]) {
			//printlnTable('skipping row: '+i+' ...');
		 	continue;
		}
		
		if (!rows1[i] || !rows2[i]) {
		  return;
		}

		var cells1 = rows1[i].cells;
		var cells2 = rows2[i].cells;
		
		if (!cells1 || !cells2) {
		  return;
		}

		//printlnTable("cells1: "+cells1[0].offsetHeight+" "+cells1[0].clientHeight);
		//printlnTable("cells2: "+cells2[0].offsetHeight+" "+cells2[0].clientHeight);
		
		var cells1OffsetHeight = cells1[0].offsetHeight;
		var cells2OffsetHeight = cells2[0].offsetHeight;
		
		if( hX_6.brw.isIE())
		{
			//in IE when the table is hidden both offsetHeights will be zero
			//in those cases we cannot compute the table row heights otherwise
			//the table will be displayed collapsed.
			if(0 == cells2OffsetHeight && 0 == cells1OffsetHeight)
			{
		 		//printlnTable("can't set rowheight for hidden tables..");
				continue; //need to continue processing and not return;
			}
		}
		
		//if (0 == cells2[0].offsetHeight) {
     		//printlnTable("cells2: "+cells2[0].offsetHeight+" "+cells2[0].clientHeight);
		//}
		
		//if (0 == cells1[0].offsetHeight) {
		 	//printlnTable("cells1: "+cells1[0].offsetHeight+" "+cells1[0].clientHeight);
		//}
		
		//alert("cells2[0].offsetHeight: " + cells2[0].offsetHeight);
		//alert("cells1[0].offsetHeight: " + cells1[0].offsetHeight);


		if (cells1OffsetHeight < MIN_HEIGHT) {
			cells1Height = MIN_HEIGHT + 'px';
			cells2Height = MIN_HEIGHT + 'px';
			
		} else {		  

		  if (cells2OffsetHeight > cells1OffsetHeight) {
			  cells1Height = cells2OffsetHeight + 'px';
			  cells2Height = cells2[0].style.height;

		  } else {
  		  cells1Height = cells1[0].style.height;
			  cells2Height = cells1OffsetHeight + 'px';
		  }		  
		}
		
		cells1[0].style.height = cells1Height;
		cells2[0].style.height = cells2Height;

	}
	
  this.exiting('HxDataTableEx.prototype.adjustRowHeights');
	return;
}

HxDataTableEx.prototype.adjustTableRowHeights = function() {
  this.entering('HxDataTableEx.prototype.adjustTableRowHeights');

	if(!this.tableId) {
	  return;
	}

	/********** row height adjustments ************/
	var tableHeader1 = hX_6.getElementById(this.tableId+"LeftColumnTBodyheader");
	var tableHeader2 = hX_6.getElementById(this.tableId+"RightColumnTBodyheader");
	var tableBody1   = hX_6.getElementById(this.tableId+"bodyDiv1");
	var tableBody2   = hX_6.getElementById(this.tableId+"bodyDiv2");
	var tableFooter1 = hX_6.getElementById(this.tableId+"LeftColumnTBodyfooter");
	var tableFooter2 = hX_6.getElementById(this.tableId+"RightColumnTBodyfooter");
	if (tableHeader1 && tableHeader2) {
		//printlnTable("*********HEADER***************");	
		this.adjustRowHeights(tableHeader1, tableHeader2);
	}
	if (tableBody1 && tableBody2) {
		//printlnTable("*********BODY***************");			
		this.refreshElement(tableBody2);
		this.refreshElement(tableBody2.parentNode);
		this.adjustRowHeights(tableBody1, tableBody2);
	}
	if (tableFooter1 && tableFooter2) {
		//printlnTable("*********FOOTER STARTS***************");
		this.adjustRowHeights(tableFooter1, tableFooter2);
		//printlnTable("*********FOOTER ENDS***************");
	}
	
  this.exiting('HxDataTableEx.prototype.adjustTableRowHeights');
	return;
}

HxDataTableEx.prototype.refreshPagerGoto = function() {  
  if (this.gotoId) {
    var pagerGoto = document.getElementById(this.gotoId);
    if (pagerGoto) {
      this.refreshElement(pagerGoto);
    }
  }
}

HxDataTableEx.prototype.enableDnD = function() {

	if(!this.tableId) {
	  return;
	}
	
	var leftHeaderRow = hX_6.getElementById(this.tableId+"headerLeftRow");
	var leftColumnHeaders = leftHeaderRow.getElementsByTagName("td");	
	
	var rightHeaderRow = hX_6.getElementById(this.tableId+"headerRightRow");
	var rightColumnHeaders = rightHeaderRow.getElementsByTagName("td");
	
	var dropTarget = new dojo.dnd.ColumnHeaderHtmlDropTarget (rightHeaderRow, ["rcolumnHeader"]);
	//var dropTarget = new dojo.dnd.HtmlDropTarget (rightHeaderRow, ["rcolumnHeader"]);
	//printlnTable("dropTarget: "+dropTarget);
	
	var x = 0;
	for (var i=0; i<rightColumnHeaders.length; i++) {
		if (rightColumnHeaders[i].parentNode == rightHeaderRow) {
			var ds = new dojo.dnd.ColumnHeaderHtmlDragSource (rightColumnHeaders[i], "rcolumnHeader");
			//var ds = new dojo.dnd.HtmlDragSource (rightColumnHeaders[i], "rcolumnHeader");
			//printlnTable("dragSource "+(x++)+": "+ds);
			continue;
		}
	}
	//printlnTable("DnD enable!");
	return;
}

 /** 
  * It must react to delta being more/less than zero.
  */
HxDataTableEx.prototype.handleOnMouseWheel = function(delta, tableId) {
	if(!tableId) {
	  return;
	}
	//alert(delta);
	var objDiv = hX_6.getElementById(tableId+"vScrollOuter");
	if(!objDiv) {
	  return;
	}
	objDiv.scrollTop -= delta*100;
	this.onVerticalScroll();
}

HxDataTableEx.prototype.OnMouseWheel = function(event){
	//printlnTable('OnMouseWheel: '+event);
	var delta = 0;
    if (!event) event = window.event;
                
    if (event.wheelDelta) {
    	delta = event.wheelDelta/120;
        if (window.opera) delta = -delta;
     } else if (event.detail) {
     	delta = -event.detail/3;
     }
     //printlnTable("delta: "+delta);
     if (delta) {
     	HxDataTableEx.prototype.handleOnMouseWheel(delta, this.id);
     }
	 if (event.preventDefault) event.preventDefault();
	 event.returnValue = false;

}

/*
 * Row highlighting
 */
HxDataTableEx.prototype.highlightRow = function(inputItem) {
  if (this.selectedRowStyle == '') {
    return;
  }

 	this.tableDom = document.getElementById(this.tableId);
	this.tableBody1Dom   = hX_6.getElementById(this.tableId+"bodyDiv1");
	this.tableBody2Dom   = hX_6.getElementById(this.tableId+"bodyDiv2");
  
  var rowIndex = this.findRowIndex(inputItem);
  this.setRowStyle(inputItem, rowIndex);
}

HxDataTableEx.prototype.findRowIndex = function(inputItem) {
	var tableBody1 = this.getTableDiv();
  var rows = tableBody1.rows;
  for (var i = 0; i < rows.length; i++) {
		var inputs = rows[i].getElementsByTagName('INPUT');
		var item = inputs[0];
		if (inputItem == item) {
			return i;
		}
	}
}

HxDataTableEx.prototype.getTableDiv = function() {
	var tablediv = this.tableBody1Dom;
	if (!tablediv){
		tablediv = this.tableBody2Dom;
	}
	return tablediv;
}

/**
 * Set the style of the row based if the inputItem is checked or not checked.
 * It also sets the inputItem title for accessibility purposes.
 */
HxDataTableEx.prototype.setRowStyle = function(inputItem, rowIndex) {

	var leftRow = null;
	if (this.tableBody1Dom) {
		leftRow = this.tableBody1Dom.rows[rowIndex];
	}
	
	var rightRow = null;
  if (this.tableBody2Dom) {
		rightRow = this.tableBody2Dom.rows[rowIndex];
	}
	
  if (leftRow && !leftRow.defaultStyle) {
    leftRow.defaultStyle = leftRow.className;
  }
  
  if(!leftRow && rightRow && !rightRow.defaultStyle) {
    rightRow.defaultStyle = rightRow.className;
  }

  if (inputItem && inputItem.checked) {
	  if (leftRow) {
		  leftRow.className = this.selectedRowStyle;
		}
		
		if (rightRow) {
		  rightRow.className = this.selectedRowStyle;
		}

		//accessibility
		if (this.selectedInputTitle) {
      inputItem.title = this.selectedInputTitle;
		}

  } else {
	  if (leftRow) {
		  leftRow.className = leftRow.defaultStyle;
		  
		  if (rightRow) {
		    rightRow.className = leftRow.defaultStyle;
		  }

		} else if (rightRow) {
		  rightRow.className = rightRow.defaultStyle;
		}

		//accessibility
		if (inputItem && this.unselectedInputTitle) {
      inputItem.title = this.unselectedInputTitle;
    }
  }

}

HxDataTableEx.prototype.highlightSelectedRows = function() {
  if (this.selectedRowStyle == '') {
    return;
  }
	this.tableDom = document.getElementById(this.tableId);
	this.tableBody1Dom   = hX_6.getElementById(this.tableId+"bodyDiv1");
	this.tableBody2Dom   = hX_6.getElementById(this.tableId+"bodyDiv2");

  var rows = null;
  if (this.tableBody1Dom) {
	  rows = this.tableBody1Dom.rows;
	} else if (this.tableBody2Dom) {
	  rows = this.tableBody2Dom.rows;
  }

  if (null != rows) {
		for (var i = 0; i < rows.length; i++) {
			var inputs = rows[i].getElementsByTagName('INPUT');
			if (inputs[0] && (inputs[0].type=="checkbox" || inputs[0].type=="radio")) {
				this.setRowStyle(inputs[0], i);
			} else {
			  //for dynamic refreshes when a selected row is removed rom the table
			  //since the updated row doesn't have a checkbox anymore it was staying selected
				this.setRowStyle(null, i);
			}
		} 
	}
}

HxDataTableEx.prototype.setInputTitles = function(selectedInputTitle, unselectedInputTitle) {
  this.selectedInputTitle = selectedInputTitle;
  this.unselectedInputTitle = unselectedInputTitle;  
}

/* Initialize mouse wheel event listener */
HxDataTableEx.prototype.initMouseWheel = function() {
	if(!this.tableId) {
	  return;
	}
	var mainDiv = hX_6.getElementById(this.tableId);
	if(window.addEventListener) mainDiv.addEventListener('DOMMouseScroll', HxDataTableEx.prototype.OnMouseWheel, false);
	window.onmousewheel = mainDiv.onmousewheel = this.OnMouseWheel;
}

HxDataTableEx.prototype.refreshElement = function(element) {
  this.entering('HxDataTableEx.prototype.refreshElement');

  if (hX_6.isPortal) {
     this.exiting('HxDataTableEx.prototype.refreshElement');
     return;
  }

  var elementStyle = element.style;

  elementStyle.visibility="visible";
  elementStyle.visibility="hidden";
  elementStyle.visibility="visible";

  //element.style.visible="visible";
  //element.style.visible="collapse";
  //element.style.visible="visible";

   elementStyle.display="";
   elementStyle.display="none";
   elementStyle.display="";
   
  this.exiting('HxDataTableEx.prototype.refreshElement');
}

HxG_6.prototype.RowSelectionController=function() {
	//printlnTable('\nRowSelectionController constructor');
	this.nSelCount=0;
	this.arrSelectionListeners=new Array();
	this.onRowSelectedFunction = null;
}

HxG_6.prototype.RowSelectionController.prototype.onAjaxComplete=function() {
	//printlnTable('\nRowSelectionController onAjaxComplete');
	this.fireSelectionListeners();
	return true;
}

HxG_6.prototype.RowSelectionController.prototype.onPageLoad=function() {
	//printlnTable('\nRowSelectionController onPageLoad');
	this.fireSelectionListeners();
	return true;
}

HxG_6.prototype.RowSelectionController.prototype.uirelease=function() {
	//printlnTable('\nRowSelectionController uirelease');
	return true;
}

HxG_6.prototype.RowSelectionController.prototype.destroy=function() {
	//printlnTable('\nRowSelectionController destroy');
	return true;
}

HxG_6.prototype.RowSelectionController.prototype.increaseCounter=function () {
	this.nSelCount++;
	//printlnTable('\nincreasing counter: '+this.nSelCount);
}

HxG_6.prototype.RowSelectionController.prototype.addSelectionListener=function (objListener) {
	if(!objListener) {
		alert("RUNTIME ERROR: null SelectionListener added to RowSelectionController");
	}
	if(!objListener.selectionChanged) {
		alert("RUNTIME ERROR: SelectionListener does not provide selectionChanged method");
	}
	/*
	if(!objListener.NO_SELECTION) {
		alert("RUNTIME ERROR: SelectionListener does not provide NO_SELECTION field");
	}
	if(!objListener.SINGLE_SELECTION) {
		alert("RUNTIME ERROR: SelectionListener does not provide SINGLE_SELECTION field");
	}
	if(!objListener.MULTIPLE_SELECTION) {
		alert("RUNTIME ERROR: SelectionListener does not provide MULTIPLE_SELECTION field");
	}
	*/			
	this.arrSelectionListeners[this.arrSelectionListeners.length]=objListener;
}

HxG_6.prototype.RowSelectionController.prototype.fireSelectionListeners=function () {

	startTime=new Date().getTime();
	var len=this.arrSelectionListeners.length;

	for(var i=0; i<len; i++) {
		var menuItemState = true;

		if (this.nSelCount==0) {
			if (this.arrSelectionListeners[i].NO_SELECTION == true) {
				menuItemState = false;
			}
		} else if (this.nSelCount==1) {
			if (this.arrSelectionListeners[i].SINGLE_SELECTION == true) {
				menuItemState = false;
			}
		} else if (this.nSelCount>1) {
			if (this.arrSelectionListeners[i].MULTIPLE_SELECTION == true) {
				menuItemState = false;
			}
		} else {
			//printlnTable('\ncontinue... selCount:'+this.nSelCount);		
			continue;
		}		
		this.arrSelectionListeners[i].selectionChanged(menuItemState);
	}
    endTime=new Date().getTime();
    //printlnTable('\on fireSelectionListeners Elapsed time '+((endTime-startTime)/1000)+' seconds.\n');	
}

HxG_6.prototype.RowSelectionController.prototype.setRowSelectedFunction=function (newFunction) {
  this.onRowSelectedFunction = newFunction;
}

HxG_6.prototype.RowSelectionController.prototype.setSelectionCount=function (newCount) {
 this.nSelCount = newCount;
}

HxG_6.prototype.RowSelectionController.prototype.selectionChanged=function (objHxSelection) {

	startTime=new Date().getTime();
	var fireAction = false;
	if(objHxSelection.isSelected()) {
		if (this.nSelCount < 2) fireAction = true;
		this.nSelCount++;
		
		if (this.onRowSelectedFunction) {
      eval(this.onRowSelectedFunction);
    }

	} else {
		if (this.nSelCount < 3) fireAction = true;	
		this.nSelCount--;
	}
	if (fireAction) {
		this.fireSelectionListeners();
	}
    endTime=new Date().getTime();
    //printlnTable('Elapsed time '+((endTime-startTime)/1000)+' seconds.');	
    //printlnTable('\nElapsed time '+((endTime-startTime)/1000)+' seconds.');
}

HxG_6.prototype.MenuRowSelectionListener=function(topMenuClientId, menuItemClientId, nSel, sSel, mSel) {
	this.NO_SELECTION=nSel;
	this.SINGLE_SELECTION=sSel;
	this.MULTIPLE_SELECTION=mSel;
	this.topMenuClientId = topMenuClientId;
	this.menuItemClientId = menuItemClientId;
}

HxG_6.prototype.MenuRowSelectionListener.prototype.selectionChanged=function (disable) {

	if (!this.menuItem || !this.menuItem.attributeTable) {
		var menu = hX_6.getComponentById(this.topMenuClientId);// Find the top-level menu
		if (!menu) {
			//printlnTable("\nmenu not found: "+menu);
			return;
		}
		this.menuitemid = menu.getMenuItemById(this.menuItemClientId);
	
		if (!this.menuitemid) {
			//printlnTable("\nmenuItemId not found");
			//printlnTable("\n**menuItemClientId: "+this.menuItemClientId);
			//printlnTable("\n**topMenuClientId: "+this.topMenuClientId);
			return;
		}

		//printlnTable("menuItemID: "+this.menuitemid+" disable: "+disable);
		this.menuItem = hX.getSubComponentById(this.menuitemid);
		//printlnTable("menuItem: "+this.menuItem);
		//printlnTable("topMenu: "+menu+" menuItemId: "+this.menuitemid+" menuItem: "+this.menuItem+" disable:"+disable);
	}
	
	if (this.menuItem) {
		
		if (disable != this.menuItem.isDisabled) {
			var setStartTime=new Date().getTime();
			if (!this.menuItem.attributeTable) {
				//printlnTable("\nNO ATTRIBUE TABLE");
				//printlnTable("\n**menuItemClientId: "+this.menuItemClientId);
				//printlnTable("\n**topMenuClientId: "+this.topMenuClientId);
				return;
			}
			this.menuItem.setAttribute("disabled:"+disable);
		  var setEndTime=new Date().getTime();
		    //printlnTable('\n......state change for: '+this.menuItemClientId);
		    //printlnTable('\n......Elapsed time '+((setEndTime-setStartTime)/1000)+' seconds.');
		}
		
		/*
		if (disable) {
			printlnTable('\ndisabling '+this.menuitemid);
			//this.menuItem.BOXobj.style.display = 'none';
			this.menuItem.BOXobj.className = 'newFaceMenuBarRoot_2_cell_disabled';
		} else {
			printlnTable('\nenabling '+this.menuitemid);
			//this.menuItem.BOXobj.style.display = 'block';
			//this.menuItem.BOXobj.className = '';			
		}
		*/
	} else {
		//printlnTable("\nmenuItem not found: "+this.menuItem);
		//printlnTable("\n??????????????????");
	}
}

/*************  ******************/
HxG_6.prototype.HxStyleRegisterEx=function(varSelection, strTableId, strSpanId, strStyles) {

	if(!hX_6.getElementById) {
		return;
	}
	this.isDescendantIndex = null;
	this.strTableId = strTableId;
	this.objTable=hX_6.getElementById(strTableId);
	if(!this.objTable) {
		alert("RUNTIME ERROR: table not found: "+strTableId);
	}
	this.objSpan=hX_6.getElementById(strSpanId);
	if(!this.objSpan) {
		alert("RUNTIME ERROR: select element not found: "+strSpanId);
	}
	this.styleIterator=new hX_6.HxStyleIterator(strStyles);
	var cellArr=this.getCellsEx();
	if(!cellArr) {
		return;
	}
	for(var i=0; i<cellArr.length; i++) {
		var td=cellArr[i];
		var style=this.styleIterator.next();
		var listener=new hX_6.HxStyleListener(td, style);
		listener.selectionChanged(varSelection);
		varSelection.addSelectionListener(listener);
	}
}

HxG_6.prototype.HxStyleRegisterEx.prototype.getCellsEx=function() {

	var result = null;

	this.objTable=hX_6.getElementById(this.strTableId+"Left");
	if(this.objTable) {
		var cellArrLeft=this.getCells();
	}

	this.objTable=hX_6.getElementById(this.strTableId+"Right");
	if(this.objTable) {
		var cellArrRight=this.getCells();
	}
	
	if (cellArrLeft && cellArrRight) {
		result = cellArrLeft.concat(cellArrRight);
	} else if (cellArrRight) {
		result = cellArrRight;
	} else {
		result = cellArrLeft;
	}
	return result;
}

HxG_6.prototype.HxStyleRegisterEx.prototype.getCells=function() {
	var tableBody=this.getChildOfType(this.objTable, hX_6._TBDY);
	if(!tableBody) {
		return null;
	}
	var trArr=this.getChildrenOfType(tableBody, hX_6._TR);
	
	if (this.isDescendantIndex != null && (trArr.length > this.isDescendantIndex)) {
		return this.getChildrenOfType(trArr[this.isDescendantIndex], hX_6._TD);
	}
	
	for(var i=0; i<trArr.length; i++) {
		if(this.isDescendant(trArr[i], this.objSpan)) {
			this.isDescendantIndex = i;
			return this.getChildrenOfType(trArr[i], hX_6._TD);
		}
	}
	return null;
}

HxG_6.prototype.HxStyleRegisterEx.prototype.getChildOfType=function(objParent, strNodeName) {
	var objArr=this.getChildrenOfType(objParent, strNodeName);
	if(objArr.length<1) {
		return null;
	}
	return objArr[0];
}

HxG_6.prototype.HxStyleRegisterEx.prototype.getChildrenOfType=function(objParent, strNodeName) {
	var objArr=new Array();
	for(var i=0; i<objParent.childNodes.length; i++) {
		if(objParent.childNodes[i].nodeName == strNodeName) {
			objArr[objArr.length]=objParent.childNodes[i];
		}
	}
	return objArr;
}

HxG_6.prototype.HxStyleRegisterEx.prototype.isDescendant=function(objParent, objChild) {
	var n=objChild;
	while(true) {
		n=n.parentNode;
		if(!n) {
			return false;
		}
		if(n == objParent) {
			return true;
		}
	}
	return false;
}

/*
function unloadDialog(thisObj, evtObj, dialogId) { 
 var dialog = hX.getComponentById(dialogId);
 if (dialog) {
//   dialog.unLoad(dialog,evtObj);
   dialog.hide(evtObj);
//   hX.i.pushOnDialogStack(dialog, true);
//   hX.i.removeFromDialogStack(dialog);//"HX_DLG_SCRATCH_"+hX_6.dialogIframeStack.length);
 }
 return true;
}
*/

// Adds the titles (i.e. tooltips) to the cells.
HxDataTableEx.prototype.addTitles = function() {
  this.entering('HxDataTableEx.prototype.addTitles');

	var sides = new Array("Left", "Right");
	for(var paneIndex=0; paneIndex < sides.length; paneIndex++) {
		var headerTBodyId = this.tableId + sides[paneIndex] + "ColumnTBodyheader";
		var headerTBody = document.getElementById(headerTBodyId);
		if(!headerTBody) {
			continue;
		}
		var headerCells = headerTBody.rows[0].cells;
		
		for(var col=0; col<headerCells.length; col++) {
			var cell = headerCells[col];
			this.ellipseColumn(cell.id, paneIndex);
		}
	}

  this.exiting('HxDataTableEx.prototype.addTitles');
}

HxDataTableEx.prototype.sizeAllHeaderColumns = function(paneIndex){
	var headerTBodyId = this.tableId 
		+ ( parseInt(paneIndex)==0? "Left" : "Right" ) 
		+ "ColumnTBodyheader";
	var headerTBody = document.getElementById(headerTBodyId);

	var headerStyle = headerTBody.parentNode.style;
	
	var cellCount = headerTBody.rows[0].cells.length-1;

	if (paneIndex == 0)
	cellCount++;
	
	var totalWidth = 0; 
	
	var widths = new Array(cellCount);
	if ((!hX_6.brw.isIE() && !hX_6.brw.isFirefox()) || (this.isFirefox2())) {
		headerStyle.tableLayout = 'auto';
	}
	
	for (var c = 0; c < cellCount; c++) {
		var headerCell = headerTBody.rows[0].cells[c];
		if (headerCell.currentWidth) {
			widths[c] = headerCell.currentWidth;
		}
		else {
			widths[c] = headerCell.offsetWidth;
		}
		totalWidth += parseInt(widths[c]);
	}
	if ((!hX_6.brw.isIE() && !hX_6.brw.isFirefox()) || (this.isFirefox2())) {
		headerStyle.display = 'none';
		headerStyle.tableLayout = 'fixed';
		headerStyle.display = '';
	}
	
	if (paneIndex == 0) {
		headerTBody.className="bodylocks";
		var mainTable = document.getElementById(this.tableId + "MainBodyTable");
        if ((!hX_6.brw.isIE() && !hX_6.brw.isFirefox()) || (this.isFirefox2())) {
			mainTable.style.tableLayout = 'auto';
        }
		var leftCol = mainTable.getElementsByTagName("colgroup")[0].getElementsByTagName("col")[0];
		leftCol.width = totalWidth;
	} else {
		headerTBody.className="bodys";
		headerTBody.parentNode.getElementsByTagName("colgroup")[0].getElementsByTagName("col")[cellCount].width = "";
	}

	for (var c = 0; c < cellCount; c++) {
		var headerCell = headerTBody.rows[0].cells[c];
	
		headerCell.currentWidth = widths[c];
		headerTBody.parentNode.getElementsByTagName("colgroup")[0].getElementsByTagName("col")[c].width = widths[c];
		try {
			var input = this.getSizerInput(headerCell);
			if( input )
				input.value = widths[c];
		} 
		catch (widthError) {
			alert("Error setting table column width: [" + widthError.name + "] [" + widthError.message + "]");
		}
	}
	
	if (paneIndex == 0) {
		var mainTable = document.getElementById(this.tableId + "MainBodyTable");
		mainTable.style.display = 'none';
        if ((!hX_6.brw.isIE() && !hX_6.brw.isFirefox()) || (this.isFirefox2())) {
			mainTable.style.tableLayout = 'fixed';
		}
		mainTable.style.display = '';
		mainTable.rows[0].cells[0].style.width = totalWidth + "px";
	}
	
	return;		
}

// For either the left or right pane of the table, iterate over all columns and
// ensure their stored widths match their actual widths on the screen.
// Adjust the split between left and right panes if the left pane is changing.
// paneIndex = 0 for left, 1 for right
HxDataTableEx.prototype.sizeAllColumns = function(paneIndex)
{
	var headerTBodyId = this.tableId 
		+ ( parseInt(paneIndex)==0? "Left" : "Right" ) 
		+ "ColumnTBodyheader";
	var mainTBodyId = this.tableId + "bodyDiv" + (parseInt(paneIndex)+1);

	var headerTBody = document.getElementById(headerTBodyId);
	var mainTBody = document.getElementById(mainTBodyId);
	if (!mainTBody) {
		//Just resize Headers and return
		this.sizeAllHeaderColumns(paneIndex);
		return;
	}
	var headerStyle = headerTBody.parentNode.style;
	var mainStyle = mainTBody.parentNode.style;
	
	var cellCount = mainTBody.rows[0].cells.length-1;
	
	// if in the left pane, we want to process all columns.  In the right pane, skip the last one--it's just a spacer.
	if (paneIndex == 0)
		cellCount++;
	var totalWidth = 0; 
	
	var widths = new Array(cellCount);

	if ((!hX_6.brw.isIE() && !hX_6.brw.isFirefox()) || (this.isFirefox2())) {
		headerStyle.tableLayout = 'auto';
		mainStyle.tableLayout = 'auto';
	}

	for (var c = 0; c < cellCount; c++) {
		// for each column...
		var headerCell = headerTBody.rows[0].cells[c];
		var mainCell = mainTBody.rows[0].cells[c];

		// if the width has been stored before (e.g. by dragging the column), then use that
		if (headerCell.currentWidth)
			widths[c] = headerCell.currentWidth;
		else
			// otherwise get the actual width on the screen.
			widths[c] = headerCell.offsetWidth;

			totalWidth += parseInt(widths[c]);
	}

	if ((!hX_6.brw.isIE() && !hX_6.brw.isFirefox()) || (this.isFirefox2())) {
		headerStyle.display = 'none';
		mainStyle.display = 'none';
		headerStyle.tableLayout = 'fixed';
		mainStyle.tableLayout = 'fixed';
		headerStyle.display = '';
		mainStyle.display = '';
	}

	if (paneIndex == 0) {
		// set the width of the left pane by using its col in the colgroup
		var mainTable = document.getElementById(this.tableId + "MainBodyTable");
        if ((!hX_6.brw.isIE() && !hX_6.brw.isFirefox()) || (this.isFirefox2())) {
			mainTable.style.tableLayout = 'auto';
		}
		var leftCol = mainTable.getElementsByTagName("colgroup")[0].getElementsByTagName("col")[0];
		leftCol.width = totalWidth;
	} else {
		// in case we are resizing for the first time, set the spacer column on the right side of the right pane to have any width
		headerTBody.parentNode.getElementsByTagName("colgroup")[0].getElementsByTagName("col")[cellCount].width = "";
		mainTBody.parentNode.getElementsByTagName("colgroup")[0].getElementsByTagName("col")[cellCount].width = "";				
	}

	// set the width of each col in the colgroups
	for (var c = 0; c < cellCount; c++) {
		var headerCell = headerTBody.rows[0].cells[c];
		var mainCell = mainTBody.rows[0].cells[c];

		headerCell.currentWidth = widths[c];
		headerTBody.parentNode.getElementsByTagName("colgroup")[0].getElementsByTagName("col")[c].width = widths[c];
		mainCell.currentWidth = widths[c];
		mainTBody.parentNode.getElementsByTagName("colgroup")[0].getElementsByTagName("col")[c].width = widths[c];

		// Send all column widths up to the server so that the table we get back
		// isn't completely different than the formatted one because that
		// causes the table to jiggle in Firefox. It also allows us to
		// keep the DOM table in synch with what the server generates so
		// that in portal we can tell if the table really changed.
		try {
			var input = this.getSizerInput(headerCell);
			if( input )
				input.value = widths[c];
		} 
		catch (widthError) {
			alert("Error setting table column width: [" + widthError.name + "] [" + widthError.message + "]");
		}
	}       

	if (paneIndex == 0) {
		// set the width of the left pane again, to the same value, but by setting its style.width
		// (I don't know why)
		var mainTable = document.getElementById(this.tableId + "MainBodyTable");
		mainTable.style.display = 'none';
        if ((!hX_6.brw.isIE() && !hX_6.brw.isFirefox()) || (this.isFirefox2())) {
			mainTable.style.tableLayout = 'fixed';
		}
		mainTable.style.display = '';
		mainTable.rows[0].cells[0].style.width = totalWidth + "px";
	}

}
HxDataTableEx.prototype.sizeAllColumnsOnAccessibleTbl = function(){
	var headerTBodyId = this.tableId +  "RightColumnTBodyheader";
	var mainTBodyId = this.tableId + "bodyDiv2" ;
	
	var headerTBody = document.getElementById(headerTBodyId);
	var mainTBody = document.getElementById(mainTBodyId);
	
	if (!mainTBody) {
		//Just resize Headers and return
		
		this.sizeAllHeaderColumnsOnAccessibleTbl();
		return;
	}
	var headerStyle = headerTBody.parentNode.style;
	var mainStyle = mainTBody.parentNode.style;
	
	
	var cellCount = mainTBody.rows[0].cells.length;
	var totalWidth = 0; 
	
	var widths = new Array(cellCount);

	if ((!hX_6.brw.isIE() && !hX_6.brw.isFirefox()) || (this.isFirefox2())) {
		headerStyle.tableLayout = 'auto';
		mainStyle.tableLayout = 'auto';
	}

	for (var c = 0; c < cellCount; c++) {
		// for each column...
		var headerCell = headerTBody.rows[0].cells[c];
		var mainCell = mainTBody.rows[0].cells[c];

		// if the width has been stored before (e.g. by dragging the column), then use that
		if (headerCell.currentWidth)
			widths[c] = headerCell.currentWidth;
		else
			// otherwise get the actual width on the screen.
			widths[c] = headerCell.offsetWidth;
			totalWidth += parseInt(widths[c]);
	}

	if ((!hX_6.brw.isIE() && !hX_6.brw.isFirefox()) || (this.isFirefox2())) {
		headerStyle.display = 'none';
		mainStyle.display = 'none';
		headerStyle.tableLayout = 'fixed';
		mainStyle.tableLayout = 'fixed';
		headerStyle.display = '';
		mainStyle.display = '';
	}
	
	for (var c = 0; c < cellCount; c++) {
		var headerCell = headerTBody.rows[0].cells[c];
		var mainCell = mainTBody.rows[0].cells[c];

		headerCell.currentWidth = widths[c];
		headerTBody.parentNode.getElementsByTagName("colgroup")[0].getElementsByTagName("col")[c].width = widths[c];
		mainCell.currentWidth = widths[c];
		mainTBody.parentNode.getElementsByTagName("colgroup")[0].getElementsByTagName("col")[c].width = widths[c];
		try {
			var input = this.getSizerInput(headerCell);
			if( input )
				input.value = widths[c];
		} 
		catch (widthError) {
			alert("Error setting table column width: [" + widthError.name + "] [" + widthError.message + "]");
		}
	}       

	return;
}


HxDataTableEx.prototype.sizeAllHeaderColumnsOnAccessibleTbl = function(){
	var headerTBodyId = this.tableId + "RightColumnTBodyheader";
	
	var headerTBody = document.getElementById(headerTBodyId);

	var headerStyle = headerTBody.parentNode.style;
	
	var cellCount = headerTBody.rows[0].cells.length;
	
	var totalWidth = 0; 
	
	var widths = new Array(cellCount);
	if ((!hX_6.brw.isIE() && !hX_6.brw.isFirefox()) || (this.isFirefox2())) {
		headerStyle.tableLayout = 'auto';
	}
	
	for (var c = 0; c < cellCount; c++) {
		var headerCell = headerTBody.rows[0].cells[c];
		if (headerCell.currentWidth) {
			widths[c] = headerCell.currentWidth;
		}
		else {
			widths[c] = headerCell.offsetWidth;
		}
		totalWidth += parseInt(widths[c]);
	}
	
	if ((!hX_6.brw.isIE() && !hX_6.brw.isFirefox()) || (this.isFirefox2())) {
		headerStyle.display = 'none';
		headerStyle.tableLayout = 'fixed';
		headerStyle.display = '';
	}
	
	
	headerTBody.className="bodylocks";
	var mainTable = document.getElementById(this.tableId + "MainBodyTable");
    if ((!hX_6.brw.isIE() && !hX_6.brw.isFirefox()) || (this.isFirefox2())) {
		mainTable.style.tableLayout = 'auto';
	}
	var leftCol = mainTable.getElementsByTagName("colgroup")[0].getElementsByTagName("col")[0];
	leftCol.width = totalWidth;
	
	
	for (var c = 0; c < cellCount; c++) {
		var headerCell = headerTBody.rows[0].cells[c];
	
		headerCell.currentWidth = widths[c];
		headerTBody.parentNode.getElementsByTagName("colgroup")[0].getElementsByTagName("col")[c].width = widths[c];
		try {
			var input = this.getSizerInput(headerCell);
			if( input )
				input.value = widths[c];
		} 
		catch (widthError) {
			alert("Error setting table column width: [" + widthError.name + "] [" + widthError.message + "]");
		}
	}
	
	return;	
}


HxDataTableEx.prototype.getSizerInput = function(headerCell) {
	// headerCell contains a table with three cells: name, sorter, and sizer. 
	// the sizer cell has the hidden input field.
	var headerTable = headerCell.getElementsByTagName("table")[0];
	if( !headerTable ) {
		return;
	}
	var sizerCell = headerTable.rows[0].getElementsByTagName("td")[2];
	if( !sizerCell ) {
		return;
	}
	return sizerCell.getElementsByTagName("input")[0];
}

// Sets the designated column to a new size.
// Called on mouseDown (mouseAction=1), mouseDrag (mouseAction=2), and mouseUp (mouseAction=3)
// event = the actual browser event
// mouseAction = my own mouseAction integer (1,2,3)
// columnId = the id of the column
// paneIndex = 0 for left pane, 1 for right
HxDataTableEx.prototype.resizeColumn = function(event, mouseAction, columnId, paneIndex)
{

    var pagePositionX = window.event ? 
                        (window.event.clientX + Math.max( document.documentElement.scrollLeft, document.body.scrollLeft ))
		: (event.pageX);
    // The above is adapted from WEvent in WClient.js.
	// IE issues with scrolling?  with RTL?
    
    var column = document.getElementById(columnId);
	
    switch (mouseAction) {
    case 1: // mouseDown (grab)
        if (!column.isDragging) {
		
			if(!this.firstResize) {
				this.firstResize = "false";
				if(this.isAccessible == true){
					this.sizeAllColumnsOnAccessibleTbl();
				}
				else {
					this.sizeAllColumns(1);
					this.sizeAllColumns(0);
				}

			}
				
            var cells = column.getElementsByTagName("table")[0].rows[0].cells;
            var minWidth = 20;

            for (var i=1; i < cells.length; i++)
                minWidth += parseInt(cells[i].width);

            column.minWidth = minWidth;
            column.isDragging = true;
            column.startPositionX = column.currentPositionX = pagePositionX;
            column.startWidth = 
            column.offsetWidth;
            // save document's mouse handlers
            document._onselectstart = document.onselectstart;
            document._onmousemove = document.onmousemove;
            document._onmouseup = document.onmouseup;
            document.body._cursor = document.body.style.cursor;
            // temporarily override document's mouse handlers
            document.onselectstart = new Function("event", "return false;");
            document.onmousemove = new Function("event", "return " + this.variableName + ".resizeColumn(event, 2, '" + columnId + "', '" + paneIndex + "');");
            document.onmouseup = new Function("event", "return " + this.variableName + ".resizeColumn(event, 3, '" + columnId + "', '" + paneIndex + "');");
            document.body.style.cursor = "e-resize";

            return false;
        }
        break;
    case 2: // mouseMove (drag)
        if (column.isDragging) {
            var mousePositionX = pagePositionX
            var difference = mousePositionX - column.startPositionX;

			if (this.isRTL) {
				difference = -difference;
			}
            column.currentWidth = Math.max(column.startWidth + difference, column.minWidth) + "px";
            column.parentNode.parentNode.parentNode.getElementsByTagName("colgroup")[0].getElementsByTagName("col")[column.cellIndex].width = column.currentWidth;
            column.currentPositionX = mousePositionX;

			// set all column sizes to match their actual widths
			if(this.isAccessible == true){
				this.sizeAllColumnsOnAccessibleTbl();
			}else{
				this.sizeAllColumns(paneIndex);                
			}
			// Reset the horizontal scrollbar in case it changed
			this.adjustHorizontalScroll(false);
			
			this.ellipseColumn(columnId, paneIndex);

            return false;
        }
        break;
    case 3: // mouseUp (drop)
        if (column.isDragging) {
            column.isDragging = false;
            // restore document's mouse handlers
            document.onselectstart = document._onselectstart;
            document.onmousemove = document._onmousemove;
            document.onmouseup = document._onmouseup;
            document.body.style.cursor = document.body._cursor;
            // save width
            var input = this.getSizerInput(column);
            input.value = column.currentWidth ? column.currentWidth : "-1";
			
			// if someone has a method named the right way to listen to columns being resized, call it
			try{
				eval( this.variableName + "ColumnSized('" + columnId + "'," + parseInt(input.value) + ");" );
			} catch(e) {
				// no big deal; just means no one is listening for column size changes
			}

            return false;
        }
        break;
    }
    
    return true;
}

// Iterate down the cells of a column (including its header), and put ellipses and a tooltip
// on all cells that are getting truncated by the column width.
// columnId = the element id of the column header cell.
// paneIndex = 0 or 1 for the left or right hand pane
HxDataTableEx.prototype.ellipseColumn = function(columnId, paneIndex)
{
  this.entering('HxDataTableEx.prototype.ellipseColumn');
  
	//do not process ellipses if not requested or
	//if the table is accessible. we do not want to
	//ellipse columns because the screen reader can't
	//read the real content.
	if((this.useEllipses == false) || (this.isAccessible == true)) {
    this.exiting('HxDataTableEx.prototype.ellipseColumn');
		return;
	}	

	if(!columnId || columnId == "") {
    this.exiting('HxDataTableEx.prototype.ellipseColumn');
		return;
	}
  
  var column = document.getElementById(columnId);
  var columnChildNodes = column.childNodes;
	
	// process the header cell
	if(columnChildNodes[0].tagName == "SPAN")
	{	  
		//header only contains a simple output component
		var width = this.calculateCellWidth(column);
		this.ellipseSpanColumnHeader(column, width);
	}
	else if( typeof columnChildNodes[0].rows != "undefined")
	{
		//header contains a table
		var cell = columnChildNodes[0].rows[0].cells[0];     
		var width = this.calculateCellWidth(cell);		
		this.ellipseSpanColumnHeader(cell, width);
	}

  // process the filter, if present
  //var headerTable = cell.parentNode.parentNode.parentNode.parentNode.parentNode.parentNode;
  var sides = new Array("Left", "Right");
  var headerTable = document.getElementById(this.tableId + sides[paneIndex] + "ColumnTBodyheader");
    if(headerTable.rows.length > 1) {
      var filterCell = headerTable.rows[1].cells[column.cellIndex];
      var width = this.calculateCellWidth(filterCell);
      this.ellipseSpanFilter(filterCell, width);
  }

	// process the cells in the column
	var paneTable = document.getElementById(this.tableId + sides[paneIndex]);
	if(!paneTable) {
		return;
	}
	var paneTBodyRows = paneTable.getElementsByTagName("tbody")[0].rows;
	var paneTBodyRowsLen = paneTBodyRows.length;
  var lastSpan;
  var width = this.calculateCellWidth(column);

	for(var rowIndex=0; rowIndex<paneTBodyRowsLen; rowIndex++) {
		cell = paneTBodyRows[rowIndex].cells[column.cellIndex];
		lastSpan = this.ellipseSpanData(cell, width, lastSpan);
	}

  this.exiting('HxDataTableEx.prototype.ellipseColumn');
}

HxDataTableEx.prototype.ellipseSpanColumnHeader = function(cell, width) {
  if (hX_6.brw.isIE()) {
    this.ellipsingHeader = true;    
  }

  this.ellipseSpan(cell, width);

  if (hX_6.brw.isIE()) {
    this.ellipsingHeader = false;   
  }
}

HxDataTableEx.prototype.ellipseSpanFilter = function(cell, width) {
  if (hX_6.brw.isIE()) {
    this.ellipsingFilter = true;
  }
  
  this.ellipseSpan(cell, width);
  
  if (hX_6.brw.isIE()) {
    this.ellipsingFilter = false;
  }
}

HxDataTableEx.prototype.ellipseSpanData = function(cell, width, lastSpan) {
  if (hX_6.brw.isIE()) {    
		this.ellipsingData = true;
  }
  
  this.ellipseSpan(cell, width, lastSpan);
  
  if (hX_6.brw.isIE()) {
		this.ellipsingData = false;
  }
}

// Analyze a td to see if its contents fit.  If not, end it with an ellipsis
// and give it a tooltip (title).  The last childNode of the cell must be a span, which
// must contain nothing but text.  Otherwise, this routine just do nothing.
HxDataTableEx.prototype.ellipseSpan = function(cell, width, lastSpan) {
	// this algorithm doesn't work on right-justified text
	if(cell.style.textAlign == "right") {
		return;
	}

	// get the span to work on, and make sure it qualifies
	var spans = cell.getElementsByTagName("span");
	if(spans.length == 0) {
		return;
	}

	var span = spans[spans.length - 1];
	var spanChildNodes = span.childNodes;	
	if(spanChildNodes.length != 1) {
		return;
	}
	if(spanChildNodes[0].nodeName != "#text") {
		return;
	}

	var textNode = spanChildNodes[0];

  if (this.useEllipsesAprox && hX_6.brw.isIE()) {
    //useEllipsesAprox turn off ellipsis while allowing the title of the cell to be set
	 if ((textNode.data.length == 1) && (0 == textNode.data.indexOf('I')) && ("hidden" == span.style.visibility)) {
	 	return span;
	 }

	cell.title = textNode.data;
    return span;
 }
	
  var realText = span.getAttribute("realText");

	// always save the real text in an attribute on the span
	if( !realText ) {
    realText = textNode.data;
    span.setAttribute("realText", realText );
	}	
	textNode.data = realText;

  var result = null;
  if (this.useEllipsesAprox && hX_6.brw.isIE()) {
    result = this.ellipseSpanAproximation(cell, width, lastSpan, textNode, span, realText);
  } else {
    result = this.ellipseSpanBinarySearch(cell, width, lastSpan, textNode, span, realText);
  }
  
  return result;
}

HxDataTableEx.prototype.ellipseSpanBinarySearch = function(cell, width, lastSpan, textNode, span, realText) {
  if ((realText.length == 1) && (0 == realText.indexOf('I')) && ("hidden" == span.style.visibility)) {
    return span;
  }
  
  width = width - span.offsetLeft;
    // If element is being shown && if we don't fit
  var offsetWidth = span.offsetWidth;

  if(offsetWidth > 0 && offsetWidth > width) 
  {
    //optimization. So we don't have to calculate ellipses all the time
    if(lastSpan && lastSpan.getAttribute("realText") == realText) 
    {
      textNode.data = lastSpan.childNodes[0].data;
      cell.title = lastSpan.parentNode.title;
      return span;
    }

    // do a binary search for the right number of characters to show
    var min = 0;
    var max = realText.length;

    while(min < max - 1) 
    {
      var chars = parseInt((max + min)/2);
      textNode.data = realText.substr(0,chars) + "...";
      offsetWidth = span.offsetWidth;
      if( offsetWidth > width ) {
        max = chars;
      } else {
        min = chars;
      }
    }

    textNode.data = realText.substr(0,min) + "...";
    cell.title = realText;
  }
  return span;
}

HxDataTableEx.prototype.getAproxRatio = function(span, textNode, realText) {

  if (typeof this.ratio == "undefined" || null == this.ratio) {

    var myDiv = document.createElement( "div" );
    myDiv.style.width = "1px";
    var mySpan = document.createElement( "span" );
    mySpan.innerHTML = realText;

    myDiv.appendChild(mySpan);
    span.parentNode.appendChild(myDiv);

    var offsetWidth = mySpan.offsetWidth;

    span.parentNode.removeChild(myDiv);

    //the higher the ratio the higher the error margin
    //increasing to have error margin
    this.ratio = (offsetWidth / realText.length) + 1;
  }
  return this.ratio;
}

HxDataTableEx.prototype.clearAproxVars = function() {
  if (typeof  this.ellipseDataOffsetLeft != "undefined") {
    this.ellipseDataOffsetLeft = null;
  }
  if (typeof this.ratio != "undefined") {
    this.ratio = null;
  }
}

HxDataTableEx.prototype.getSpanOffsetLeft = function(span) {
  
  var realTextOffsetLeft = null;

  if (this.ellipsingData) {
    
    if (typeof  this.ellipseDataOffsetLeft == "undefined" || null == this.ellipseDataOffsetLeft) {
      //only two type of columns
      //text only
      //icon plus text
      this.ellipseDataOffsetLeft = new Array(2);      
    }    
    
    var i;
    if (null == span.previousSibling) {
      i = 0;    
    } else {
      i = 1; 
    }
    
    if (this.ellipseDataOffsetLeft[i] == "undefined" || null == this.ellipseDataOffsetLeft[i]) {
      realTextOffsetLeft = span.offsetLeft;
      this.ellipseDataOffsetLeft[i] = realTextOffsetLeft;
      
    } else {
      realTextOffsetLeft = this.ellipseDataOffsetLeft[i];
    }

  } else if (this.ellipsingHeader) {
    
    if (typeof  this.ellipseHeaderOffsetLeft == "undefined") {
      realTextOffsetLeft = span.offsetLeft;
      this.ellipseHeaderOffsetLeft = realTextOffsetLeft;
      
    } else {
      realTextOffsetLeft = this.ellipseHeaderOffsetLeft;
    }

  } else if (this.ellipsingFilter) {
    
    if (typeof  this.ellipseFilterOffsetLeft == "undefined") {
      realTextOffsetLeft = span.offsetLeft;
      this.ellipseFilterOffsetLeft = realTextOffsetLeft;
      
    } else {
      realTextOffsetLeft = this.ellipseFilterOffsetLeft;
    }
  }
  
  return realTextOffsetLeft;
  
}

HxDataTableEx.prototype.ellipseSpanAproximation = function(cell, width, lastSpan, textNode, span, realText) {
  
  if ((realText.length == 1) && (0 == realText.indexOf('I')) && ("hidden" == span.style.visibility)) {
    return span;
  }
  
  // NOTE: 119381 - aproximation was leaving too much white space
  // NOT trying to use ellipsis anymore instead setting realtext and the cell title  
  
//  var realTextOffsetLeft = this.getSpanOffsetLeft(span);
//  width = width - realTextOffsetLeft;
//  // If element is being shown && if we don't fit
//  //aproximating don't care about the exact width
//  //var offsetWidth = span.offsetWidth;
//
//	var ratio = this.getAproxRatio(span, textNode, realText);
//	var aproxW = realText.length * ratio;
//	if (aproxW >= width) {
//	  //aprox
//	  //substract 3 chars because they are being replaced by ...
//	  var max = parseInt(width / ratio) - 3;
//	  textNode.data = realText.substr(0, max) + "...";    
//	}
  cell.title = realText;  
  return span;
}

HxDataTableEx.prototype.setUseEllipsesAprox = function(useEllipsesAprox) {
  this.useEllipsesAprox = useEllipsesAprox;
}

HxDataTableEx.prototype.calculateCellWidth = function(cell)
{

    if (this.useEllipsesAprox && hX_6.brw.isIE()) {
      //useEllipsesAprox turn off ellipsis while allowing the title of the cell to be set
      return 0;
    }
  
	var style;
	if(window.getComputedStyle)
	{
		style = window.getComputedStyle(cell, null);
	}
	else
	{
		style = cell.currentStyle;
	}
		
    var width = cell.clientWidth - parseInt(style.paddingRight);

    if( !hX_6.brw.isIE() )
    {
        width -= parseInt(style.paddingLeft);
    }
    
    return width;
}

HxDataTableEx.prototype.adjustScrollOnEmptyData = function(){
	//Adjust horizantal scroll
	var outerScroll = hX_6.getElementById(this.tableId+'hScrollOuter');
	var innerScroll = hX_6.getElementById(this.tableId+'hScrollInner');

	var diff = 0;
	var tableHeaderDivLeft = hX_6.getElementById(this.tableId+'columnheaderLeftCellsDiv');	
	var tableHeaderDivRight = hX_6.getElementById(this.tableId+'columnheaderRightCellsDiv');
	if (tableHeaderDivRight) {
		diff = tableHeaderDivRight.scrollWidth - tableHeaderDivRight.offsetWidth;
		outerScroll.parentNode.parentNode.style.display = "";
		var wi = 0;
		if (tableHeaderDivLeft) {
			wi += tableHeaderDivLeft.offsetWidth;
		}

		if (tableHeaderDivRight) {
			wi += tableHeaderDivRight.offsetWidth;
		}
		outerScroll.style.width = (wi + "px");
		innerScroll.style.width = ((wi + diff ) + "px");
		var scrollbarWidth = this.getScrollbarWidth(tableHeaderDivRight);
		outerScroll.style.height = scrollbarWidth + 'px';
		innerScroll.style.height = scrollbarWidth + 'px';
		if (!this.hideScrollbar && diff == 0) {
			outerScroll.style.overflowX = "scroll";
			outerScroll.style.overflowY = "hidden";
			innerScroll.style.width = 0 + "px";
		}

	}

	return;
}

HxDataTableEx.prototype.isFirefox2 = function(){
  
  return (navigator.userAgent.indexOf('Firefox/2.')>-1);
}

HxDataTableEx.prototype.clearSizerInput = function(){

	//Runt this for both left and right side columns
	for(var paneIndex =0; paneIndex <= 1; paneIndex++){
		var headerTBodyId = this.tableId+(parseInt(paneIndex)==0? "Left" : "Right" )+ "ColumnTBodyheader";
		var headerTBody = document.getElementById(headerTBodyId);

		if(!headerTBody){
			this.clearSizerInputOnAccessibleTbl();
			return;
		}

		var cellCount = headerTBody.rows[0].cells.length-1;
		if (paneIndex == 0)
		 cellCount++;

		for (var c = 0; c < cellCount; c++) {
			var headerCell = headerTBody.rows[0].cells[c];
			var headerTable = headerCell.getElementsByTagName("table")[0];
			if (!headerTable)
			continue;
			var sizerCell = headerTable.rows[0].getElementsByTagName("td")[2];
			if (!sizerCell)
			continue;
			var input =     sizerCell.getElementsByTagName("input")[0];
			if( input )
			   input.value = "-1";
		}
	}
}

HxDataTableEx.prototype.clearSizerInputOnAccessibleTbl = function(){
	var headerTBodyId = this.tableId + "RightColumnTBodyheader";
	var mainTBodyId = this.tableId + "bodyDiv2";
	var headerTBody = document.getElementById(headerTBodyId);
	var mainTBody = document.getElementById(mainTBodyId);

	if(!headerTBody){
		return;
	}

	var cellCount = headerTBody.rows[0].cells.length;

	for (var c = 0; c < cellCount; c++) {
		var headerCell = headerTBody.rows[0].cells[c];
		var headerTable = headerCell.getElementsByTagName("table")[0];
		if (!headerTable)
			continue;
		var sizerCell = headerTable.rows[0].getElementsByTagName("td")[2];
		if (!sizerCell)
			continue;
		var input =     sizerCell.getElementsByTagName("input")[0];
		if( input )
		   input.value = "-1";
	}

}
/*
HxDataTableEx.prototype.destroy = function(){
  //do nothing - place holder - in case something needs to be destroy from this component
  //this function is called by JWL when removing a component from the DOM
}
*/

HxDataTableEx.prototype.entering = function(methodName) {
  //do nothing - place holder - for tracing or performance measurements
}

HxDataTableEx.prototype.exiting = function(methodName) {
  //do nothing - place holder - for tracing or performance measurements
}

HxDataTableEx.prototype.gotoPageAction = function() {
  var tableAction = document.getElementById(this.tableId + ":tableAction");
  var gotoText = document.getElementById(this.gotoId + "__pagerGoText");
  if (null != tableAction && null != gotoText && null != gotoText.value && gotoText.value.length > 0) {
    tableAction.value='goto:'+gotoText.value;
  }
}
